import regex as re

SENTENCE_SEPARATOR = ["។", "៕", "?", "!"]
RE_SENTENCE_SEPARATOR = re.compile("(?<=[" + "".join(SENTENCE_SEPARATOR) + "])")
RE_REPEATING_PUNCT = re.compile(r"([?!៕។៖]){2,}")

def split_on_punctuations(text: str):
  values = RE_SENTENCE_SEPARATOR.split(text)
  if values[-1]:
    return values
  return values[:-1]

def processor(text: str) -> str:
  sentences = []
  for line in text.splitlines():
    line = line.strip()
    if not line:
      continue
    line = RE_REPEATING_PUNCT.sub(r"\1", line)
    for sentence in split_on_punctuations(line):
      sentences.append(sentence)
  return sentences



# -*- coding: utf-8 -*-
import regex as re
from tha.strings import overwrite_spans

RE_ONLY_TIME = re.compile(
  r"([0-9\u17e0-\u17e9]{1,2}):([0-9\u17e0-\u17e9]{2})\s?(am|pm|AM|PM)?"
)

RE_DATE_YEAR_FIRST = re.compile(
  r"([\u17e0-\u17e9\d]{4})([/-])([\u17e0-\u17e9\d]{2})\2([\u17e0-\u17e9\d]{2})"
)

RE_DATE_DAY_FIRST = re.compile(
  r"([\u17e0-\u17e9\d]{2})([/-])([\u17e0-\u17e9\d]{2})\2([\u17e0-\u17e9\d]{4})"
)


def date_processor(text: str) -> str:
  replacements = []

  for m in RE_DATE_YEAR_FIRST.finditer(text):
    replacement = f"{m[1]} {m[3]} {m[4]}"
    replacements.append((m.start(), m.end(), replacement))

  for m in RE_DATE_DAY_FIRST.finditer(text):
    replacement = f"{m[1]} {m[3]} {m[4]}"
    replacements.append((m.start(), m.end(), replacement))

  return overwrite_spans(text, replacements)


def time_processor(text: str) -> str:
  replacements = []
  for m in RE_ONLY_TIME.finditer(text):
    replacement = f"{m[1]} {m[2]}"
    if m[3] is not None:
      replacement += "▁" + "▁".join(m[3])
    replacements.append((m.start(), m.end(), replacement))
  return overwrite_spans(text, replacements)



import regex as re
from urlextract import URLExtract

domain_lookup = {
  "kh": "K▁H",
  "gmail": "g▁mail",
  "edu": "E▁D▁U",
  "org": "O▁R▁G",
  "gov": "gov",
  "per": "P▁E▁R",
  "io": "I▁O",
  "us": "U▁S",
  "sg": "S▁G",
  "ru": "R▁U",
  "th": "T▁H",
}

url_extractor = URLExtract(extract_email=True, extract_localhost=True)


def url_verbalize(url: str) -> str:
  url = re.sub(r"https?:\/\/", "", url, re.IGNORECASE)
  paths = url.split(".")
  return " dot ".join(
    map(
      lambda x: domain_lookup[x.lower()].lower() if x.lower() in domain_lookup else x,
      paths,
    )
  )


def email_verbalize(email: str) -> str:
  username, domain = email.split("@")
  return f"{username} at {url_verbalize(domain)}"


def processor(text: str) -> str:
  if url_extractor.has_urls(text):
    for url in url_extractor.gen_urls(text):
      if "@" in url:
        text = text.replace(url, email_verbalize(url))
        continue
      text = text.replace(url, url_verbalize(url))
  return text



import regex as re

RE_PLATE_NUMBER = re.compile(r"([0-9០-៩]+[A-Z]+)[\- ]([0-9០-៩]{4,})")
RE_PLATE_SQUARE_NUMBER = re.compile(r"([0-9០-៩])\1{3}")


def _replacer(m):
  sep = "▁"
  left = " ".join(m[1])
  right = (
    f"ការ៉េ{m[2][0]}"
    if RE_PLATE_SQUARE_NUMBER.search(m[2])
    else m[2][:2] + sep + m[2][2:]
  )
  return f"{left} {right}"


def processor(text):
  return RE_PLATE_NUMBER.sub(_replacer, text)



import regex as re
from tha.cardinals import processor as cardinals_processor

RE_ORDINALS = re.compile(r"([0-9]+)(st|nd|rd|th)")


def processor(text: str, **kwargs) -> str:
  if not RE_ORDINALS.search(text):
    return text
  return cardinals_processor(RE_ORDINALS.sub(r"ទី▁\1", text), **kwargs)



import regex as re
import math

DEFAULT_MINUS = "ដក"
DEFAULT_DELIMITER = "ក្បៀស"
DEFAULT_SEPARATOR = "▁"

digits = [
  "សូន្យ",
  "មួយ",
  "ពីរ",
  "បី",
  "បួន",
  "ប្រាំ",
  "ប្រាំមួយ",
  "ប្រាំពីរ",
  "ប្រាំបី",
  "ប្រាំបួន",
]

prefix = [
  "",
  "ដប់",
  "ម្ភៃ",
  "សាមសិប",
  "សែសិប",
  "ហាសិប",
  "ហុកសិប",
  "ចិតសិប",
  "ប៉ែតសិប",
  "កៅសិប",
]

suffix = dict(
  [
    (2, "រយ"),
    (3, "ពាន់"),
    (4, "ម៉ឺន"),
    (5, "សែន"),
    (6, "លាន"),
    (9, "ប៊ីលាន"),
    (12, "ទ្រីលាន"),
    (15, "ក្វាទ្រីលាន"),
    (18, "គ្វីនទីលាន"),
    (21, "សិចទីលាន"),
    (24, "សិបទីលាន"),
    (27, "អុកទីលាន"),
    (30, "ណូនីលាន"),
    (33, "ដេស៊ីលាន"),
    (36, "អាន់ដេស៊ីលាន"),
  ]
)


def cardinal_processor(num, sep=DEFAULT_SEPARATOR, minus_sign=DEFAULT_MINUS):
  if math.isnan(num):
    return ""

  if num < 0:
    return minus_sign + sep + cardinal_processor(abs(num), sep)

  num = math.floor(num)

  if num < 10:
    return digits[num]

  if num < 100:
    r = num % 10
    if r == 0:
      return prefix[num // 10]
    return prefix[num // 10] + cardinal_processor(r, sep)

  exp = math.floor(math.log10(num))

  while exp not in suffix and exp > 0:
    exp = exp - 1

  d = 10**exp
  pre = cardinal_processor(num // d, sep)
  suf = suffix[exp]
  r = num % d

  if r == 0:
    return pre + suf

  return pre + suf + sep + cardinal_processor(r, sep)


RE_CARDINALS = re.compile(r"[\-+]?[\u17e0-\u17e90-9]+")


def processor(text: str, sep=DEFAULT_SEPARATOR, minus_sign=DEFAULT_MINUS) -> str:
  return RE_CARDINALS.sub(
    lambda m: cardinal_processor(int(m[0]), sep=sep, minus_sign=minus_sign), text
  )



import regex as re
from tha.cardinals import processor as cardinals_processor

RE_DECIMALS = re.compile(r"([\-]?[\u17e0-\u17e90-9]+)([\.\,])([\u17e0-\u17e90-9]+)")
RE_NUM_LEADING_ZEROS = re.compile(
  r"([\-]?[\u17e0-\u17e90-9]+)([\.\,])([\u17e00]+)([\d\u17e0-\u17e9]+)"
)

def processor(text: str, comma_word="ក្បៀស", point_word="ចុច") -> str:
  translation = str.maketrans({".": f"▁{point_word}▁", ",": f"▁{comma_word}▁"})

  def leading_zeros_replacer(m):
    return f"{m[1]}{m[2]}{'▁'.join(m[3])}▁{m[4]}".translate(translation)

  def decimals_replacer(m):
    sep = m[2].translate(translation)
    return f"{m[1]}{sep}{m[3]}"

  text = RE_NUM_LEADING_ZEROS.sub(leading_zeros_replacer, text)
  text = RE_DECIMALS.sub(decimals_replacer, text)
  text = cardinals_processor(text)

  return text



import regex as re
from tha.strings import overwrite_spans
from tha.cardinals import cardinal_processor, processor as cardi_processor
from tha.decimals import processor as decimals_processor
from tha.verbatim import verbatim_dict

verbtim_translator = str.maketrans(verbatim_dict)

RE_NUM_CURRENCY = re.compile(
  r"([\$€£៛₫₽¥₩฿₭])\s?([\d\u17e0-\u17e9]+\.?[\d\u17e0-\u17e9]*)"
)

RE_MONEY_LEADING_ZEROS = re.compile(r"^[0\u17e0]+")
RE_MONEY_USD_DECIMAL = re.compile(
  r"\$ ?(([\d\u17e0-\u17e9]+,)*[\d\u17e0-\u17e9]+)(\.[\d\u17e0-\u17e9]+)?|(([\d\u17e0-\u17e9]+,)*[\d\u17e0-\u17e9]+)(\.[\d\u17e0-\u17e9]+)? ?\$"
)


def reoder(text: str) -> str:
  return RE_NUM_CURRENCY.sub(r"\2▁\1", text)


def currency_usd(
  text: str, currency_text="ដុល្លារ", cent_text="សេន", separator="▁"
) -> str:
  replacements = []
  max_precision = 2
  for m in RE_MONEY_USD_DECIMAL.finditer(text):
    cadinal = m[4] or m[1]
    floating_point = m[6] or m[3]
    if floating_point is None or len(floating_point) > max_precision + 1:
      continue

    cadinal = cadinal.replace(",", "")
    cadinal = RE_MONEY_LEADING_ZEROS.sub("", cadinal)
    floating_point = floating_point[1:]

    if len(floating_point) == 1:
      floating_point += "0" * (max_precision - len(floating_point))  # pad zeros

    floating_point = RE_MONEY_LEADING_ZEROS.sub("", floating_point)
    text_segments = []

    if cadinal:
      text_segments.append(cardinal_processor(int(cadinal)) + currency_text)

    if floating_point:
      text_segments.append(cardinal_processor(int(floating_point)) + cent_text)

    # zero
    if not floating_point and not cadinal:
      text_segments.append(cardinal_processor(int(0)) + currency_text)

    replacements.append((m.start(), m.end(), separator.join(text_segments)))
  return overwrite_spans(text, replacements)


def processor(text: str):
  text = currency_usd(text)
  text = reoder(text)
  text = decimals_processor(text)
  text = cardi_processor(text)
  return text.translate(verbtim_translator)



def overwrite_spans(text, replacements):
  replacements.sort(reverse=True)
  for start, end, replacement in replacements:
    start = max(start, 0)
    end = min(end, len(text))
    text = text[:start] + replacement + text[end:]
  return text



import setuptools

with open("README.md", "r") as f:
  long_description = f.read()

setuptools.setup(
  name="tha",
  version="0.1.2",
  description="A Khmer Text Normalization and Verbalization Toolkit.",
  long_description=long_description,
  long_description_content_type="text/markdown",
  url="https://github.com/seanghay/tha",
  author="Seanghay Yath",
  author_email="seanghay.dev@gmail.com",
  license="Apache License 2.0",
  classifiers=[
    "Programming Language :: Python :: 3",
    "Operating System :: OS Independent",
    "Intended Audience :: Developers",
    "Natural Language :: English",
  ],
  python_requires=">3.5",
  packages=setuptools.find_packages(),
  package_dir={"tha": "tha"},
  install_requires=[
    "urlextract",
    "phonenumbers",
    "regex",
    "ftfy",
  ],
)



import regex as re
from phonenumbers import PhoneNumberMatcher
from tha.strings import overwrite_spans

RE_NON_NUMBER = re.compile(r"[^\d+]+")


def processor(text: str, chunk_size=2, delimiter="▁", country_code="KH") -> str:
  replacements = []
  for m in PhoneNumberMatcher(text, country_code):
    phone_number = str(m.number.national_number)
    carrier_code = phone_number[:2]
    phone_number_id = phone_number[2:]
    i = 0
    chunks = []
    while i < len(phone_number_id):
      c = phone_number_id[i : i + chunk_size]
      if len(c) == chunk_size:
        chunks.append(c)
      else:
        chunks[-1] += c
      i += chunk_size
    digits = []
    for chunk in chunks:
      for i, c in enumerate(chunk):
        digits.append(c)
        if c != "0":
          digits[-1] = chunk[i:]
          break
    normalized = delimiter.join(digits)
    result = f"0{delimiter}{carrier_code}{delimiter}{normalized}"
    replacements.append((m.start, m.end, result))
  return overwrite_spans(text, replacements)



import regex as re


def _override(text, start, end, replacement):
  chunk_start = text[0:start]
  chunk_end = text[end:]
  text = chunk_start + replacement + chunk_end
  return text


def _extended_word_tokenize(text, tokenizer):
  items = []
  for m in re.finditer(r"([^\u1780-\u17ff\s]+)|([\u1780-\u17ff\s]+)", text):
    if m.group(1):
      items.append(m.group(1))
      continue

    for item in tokenizer(m.group(2)):
      whitelisted_items = ["ឯទៀត"]
      skipped = False
      for whitelist in whitelisted_items:
        if item.endswith(whitelist):
          items.append(item[0 : len(whitelist)])
          items.append(whitelist)
          skipped = True

      if skipped:
        continue

      items.append(item)
      continue

  return items


def processor(text, tokenizer, sep="▁"):
  if not re.search(r"ៗ", text):
    return text

  index_offset = 0
  cloned_text = text

  for match in re.finditer(r"ៗ+", cloned_text):
    offset_text = text[0 : match.start() + index_offset]
    tokens = _extended_word_tokenize(offset_text, tokenizer)
    tokens = tokens[-3:]
    replacement = None

    if len(tokens) <= 1:
      replacement = tokens[0]

    if len(tokens) > 1:
      if tokens[-2] in set(["1", "មួយ", "ទាំង", "លើក"]):
        replacement = "".join(tokens[-2:])

      if tokens[-1] in set(["ឡើង", "ទៅ", "ម្ដង", "ទៀត"]):
        if tokens[-2] in set(["ម៉ោង", "ថ្ងៃ", "ខែ", "ឆ្នាំ", "សប្ដាហ៍", "អាទិត្យ", "យប់"]):
          replacement = "".join(tokens[-3:])
        elif tokens[-2] in set(["ពេល"]):
          replacement = tokens[-1]
        else:
          replacement = "".join(tokens[-2:])

      if tokens[-2] == "ម្នាក់" and tokens[-1] == "ម្ដង":
        replacement = "".join(tokens[-2:])

      if tokens[-1] == "ម្នាក់" and tokens[-2] == "ម្ដង":
        replacement = "".join(tokens[-2:])

    # fallback to one word
    if replacement is None:
      replacement = tokens[-1]

    text = _override(
      text,
      match.start() + index_offset,
      match.end() + index_offset,
      f"{sep}{replacement}",
    )

    index_offset += len(replacement) - (match.end() - match.start()) + 1
  return text
